/* EPITA - IP - S3/4 */
/* question15 */

#define QNUM 15
#define QOPT "graine taille"
#define QNOPT 2

# include <assert.h>
# include <stdio.h>
# include <stdlib.h>
# include <unistd.h>

# include "base_test.h"
# include "question15.h"

static
struct tree* build_bst(size_t depth, int mink, int maxk, struct tree *store, size_t *idx) {
  struct tree *tree = store + *idx;
  *idx += 1;
  tree->key = 1 + mink + (maxk - mink)/3 + (random() % ((maxk - mink) / 4));
  tree->left = tree->right = NULL;
  if (depth) {
    depth -= 1;
    tree->left = build_bst(depth, mink, tree->key, store, idx);
    tree->right = build_bst(depth, tree->key, maxk, store, idx);
  }
  return tree;
}

static
void print_tree(struct tree *tree, int pad) {
  if (tree) {
    printf("%*s<%d", pad, "", tree->key);
    if (tree->left) {
      printf("\n");
      print_tree(tree->left, pad + 2);
    }
    if (tree->right) {
      printf("\n");
      print_tree(tree->right, pad + 2);
    }
    if (tree->left != tree->right)
      printf("\n%*s", pad, "");
    printf(">");
  }
}

static inline
void test_found(struct tree *node) {
  if (node)
    printf("found %p (key: %d)", node, node->key);
  else
    printf("not found");
}

static
void check_inval(struct tree* tree, struct tree *store, size_t len) {
  size_t pos = random() % len;
  int key = (store + pos)->key;
  printf("bst_find(tree, %d) = ", key);
  struct tree *node = bst_find(tree, key);
  test_found(node);
  printf(" \t[%s]\n", check(node == store + pos));
}

static
void check_outval(struct tree *tree, int maxk) {
  int key = maxk + random() % maxk;
  printf("bst_find(tree, %d) = ", key);
  struct tree *node = bst_find(tree, key);
  test_found(node);
  printf("\n");
}

static
void check_random_val(struct tree *tree, int mink, int maxk) {
  int key = mink + random() % maxk;
  printf("bst_find(tree, %d) = ", key);
  struct tree *node = bst_find(tree, key);
  test_found(node);
  printf("\n");
}

static
void random_test(size_t len) {
  printf("Random Tests:\n");
  struct tree *store = calloc(1<<(len+1), sizeof (struct tree));
  struct tree *tree;
  int bound = 1 << (len + 3);
  size_t idx = 0;
  tree = build_bst(len, 0, bound, store, &idx);
  printf("tree =\n");
  print_tree(tree, 2);
  printf("\n");
  check_inval(tree, store, idx);
  check_outval(tree, bound);
  check_random_val(tree, 0, bound);
  free(store);
}

int main(int argc,char *argv[])
{
  size_t        size;

  help_asked(argc,argv,QNUM,QOPT,QNOPT);

  if (argc <= QNOPT)
    {
      fprintf(stderr, "Parametres insuffisants.\n");
      usage(NULL,QNUM,QOPT);
    }
  else
    {
      seedInit(argv);
      size = getSize(argv);
      random_test(size);
    }
  exit(0);
}
